﻿using System;
using System.Collections.Generic;
using EnvDTE;
using EnvDTE80;

namespace VsTortoiseSVN.Commands
{
    public abstract class ActiveProjectBase : CommandBase
    {
        /// <summary>
        /// Gets a list with all filenames included in the current selection.
        /// </summary>
        protected List<string> GetDirectoryNames()
        {
            List<string> fullNames = new List<string>();

            Project project = ActiveProject;
            if (project != null)
            {
                project.GetDirectoryNames(fullNames);
                PathHelper.RemoveSubDirectories(fullNames);
            }

            return fullNames;
        }

        /// <summary>
        /// Gets the active project or null when none is available.
        /// Active project is, if an active document exists, the project that contains
        /// this active document or if no active document exists, the project selected in the solution explorer.
        /// </summary>
        protected Project ActiveProject
        {
            get
            {
                // During real-world usage, testers reported the
                // active project should actually depend on in what
                // document someone is. The project of the active
                // document is wanted, rather than the selected
                // from the solution explorer.
                if (Application.HasDocument())
                {
                    ProjectItem projectItem = Application.ActiveDocument.ProjectItem;
                    if (projectItem != null && projectItem.ContainingProject != null)
                    {
                        return projectItem.ContainingProject;
                    }
                }

                // No active document available, use the real
                // active project instead.
                Project project = Application.ActiveProject();
                return project;
            }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public ActiveProjectBase(DTE2 application, string name, string caption, string tooltip, int bitmapId)
            : base(application, name, caption, tooltip, bitmapId)
        {
        }
    }
}
